(* Lower level interface *)

open Unix
open Support

exception TkError of string
      (* Raised by the communication functions *)

val cTKtoCAMLwidget : string -> widget
      (* Conversion functions *)

val debug : bool ref 
      (* When set to true, displays approximation of intermediate Tcl code *)

type callback_buffer = string list
      (* Buffer for reading callback arguments *)

type tkArgs =
    TkToken of string
  | TkTokenList of tkArgs list		(* to be expanded *)
  | TkQuote of tkArgs			(* mapped to Tcl list *)

type cbid

external splitlist : string -> string list
      	= "camltk_splitlist"

val callback_naming_table : (cbid, callback_buffer -> unit) Hashtblc.t
val callback_memo_table : (widget, cbid) Hashtblc.t
      (* Exported for debug purposes only. Don't use them unless you
         know what you are doing *)
val new_function_id : unit -> cbid
val string_of_cbid : cbid -> string
val register_callback : widget -> (callback_buffer -> unit) -> string
      (* Callback support *)
val clear_callback : cbid -> unit
      (* Remove a given callback from the table *)
val remove_callbacks: widget -> unit
      (* Clean up callbacks associated to widget. Must be used only when
      	 the Destroy event is bind by the user and masks the default
	 Destroy event binding *)

val add_destroy_hook : (widget -> unit) -> unit

val   openTk : unit -> widget
val   openTkClass: string -> widget
val   openTkDisplayClass: string -> string -> widget
val   closeTk : unit -> unit
val   mainLoop : unit -> unit
val   add_fileinput : file_descr -> (unit -> unit) -> unit
val   remove_fileinput: file_descr -> unit
val   add_fileoutput : file_descr -> (unit -> unit) -> unit
val   remove_fileoutput: file_descr -> unit
      (* see [tk] module *)
val   tkEval : tkArgs array -> string
val   tkreturn: string -> unit

type timer

val  add_timer : int -> (unit -> unit) -> timer
val  remove_timer : timer -> unit

val var_handle : string -> (unit -> unit) -> unit
